datablock StaticShapeData(GridShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Grid.dts";
};

datablock StaticShapeData(CameraActiveShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Camera_Active.dts";
};

datablock StaticShapeData(CameraInactiveShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Camera_Inactive.dts";
};

datablock StaticShapeData(ConeSelectShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Select_Cone.dts";
};

datablock StaticShapeData(ConeShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_Cone.dts";
	wireframe = ConeSelectShape;
};
datablock StaticShapeData(ConeSmoothShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_Cone_Smooth.dts";
};
ConeShape.smoothData = ConeSmoothShape;
ConeSmoothShape.flatData = ConeShape;

datablock StaticShapeData(CubeSelectShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Select_Cube.dts";
};

datablock StaticShapeData(CubeShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_Cube.dts";
	wireframe = CubeSelectShape;
};

datablock StaticShapeData(CylinderSelectShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Select_Cylinder.dts";
};

datablock StaticShapeData(CylinderShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_Cylinder.dts";
	wireframe = CylinderSelectShape;
};
datablock StaticShapeData(CylinderSmoothShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_Cylinder_Smooth.dts";
};
CylinderShape.smoothData = CylinderSmoothShape;
CylinderSmoothShape.flatData = CylinderShape;

datablock StaticShapeData(PlaneSelectShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Select_Plane.dts";
};

datablock StaticShapeData(PlaneShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_Plane.dts";
	wireframe = PlaneSelectShape;
};

datablock StaticShapeData(UVSphereSelectShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Select_UVSphere.dts";
};

datablock StaticShapeData(UVSphereShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_UVSphere.dts";
	wireframe = UVSphereSelectShape;
};
datablock StaticShapeData(UVSphereSmoothShape)
{
	shapeFile = $CurrMod_BasePath @ "Models/Shape_UVSphere_Smooth.dts";
};
UVSphereShape.smoothData = UVSphereSmoothShape;
UVSphereSmoothShape.flatData = UVSphereShape;

function createDefaultScene()
{
	if (isObject(BlenderScene))
	{
		BlenderScene.grid.delete();
		BlenderScene.objects.delete();
		BlenderScene.wireframes.delete();
		BlenderScene.delete();
	}
	new ScriptObject(BlenderScene)
	{
		class = "BlenderSceneSO";
	};
	BlenderScene.objects = new SimGroup();
	BlenderScene.wireframes = new SimGroup();
	BlenderScene.objects.scene = BlenderScene;
	BlenderScene.tool = createBlenderTool();
	BlenderScene.tool.scene = BlenderScene;
	BlenderScene.tool.hideNode("ALL");
	%grid = new StaticShape()
	{
		datablock = GridShape;
		scale = "2 2 2";
	};
	%grid.setNodeColor("GridFloor", "0.29 0.29 0.29 1");
	%grid.setNodeColor("GridXAxis", "0.518 0.086 0.086 1");
	%grid.setNodeColor("GridYAxis", "0.086 0.518 0.086 1");
	%grid.setNodeColor("GridZAxis", "0.086 0.086 0.518 1");
	%grid.hideNode("GridZAxis");
	BlenderScene.grid = %grid;
	BlenderScene.loadDefault();
}

function createEmptyScene()
{
	%scene = new ScriptObject()
	{
		class = "BlenderSceneSO";
	};
	%scene.objects = new SimGroup();
	%scene.wireframes = new SimGroup();
	%scene.objects.scene = %scene;
	%scene.tool = createBlenderTool();
	%scene.tool.scene = %scene;
	%scene.tool.hideNode("ALL");
	return %scene;
}

function BlenderSceneSO::createCamera(%this, %pos, %rot, %dontSelect)
{
	if (!%dontSelect && isObject(%this.tool) && isObject(%this.tool.user))
		return;
	%data = isObject(%this.activeCamera) ? CameraInactiveShape : CameraActiveShape;
	%cam = new StaticShape()
	{
		datablock = %data;
		scale = "2 2 2";
	};
	%cam.wireframe = %cam;
	%cam.setNodeColor("ALL", "0 0 0 1");
	%cam.setTransform(vectorScale(%pos, 2) SPC %rot);
	%cam.shapePosition = %cam.getPosition();
	%cam.shapeRotation = getWords(%cam.getTransform(), 3, 6);
	%cam.shapeScale = %cam.getScale();
	if (!isObject(%this.activeCamera))
		%this.activeCamera = %cam;
	%this.objects.add(%cam);
	if (%dontSelect)
		return %cam;
	%this.selectAll(-1);
	%this.select(%cam);
	return %cam;
}

function BlenderSceneSO::createCone(%this, %pos, %rot, %scale, %dontSelect)
{
	return %this.createPrimitive(ConeShape, %pos, %rot, %scale, %dontSelect);
}

function BlenderSceneSO::createCube(%this, %pos, %rot, %scale, %dontSelect)
{
	return %this.createPrimitive(CubeShape, %pos, %rot, %scale, %dontSelect);
}

function BlenderSceneSO::createCylinder(%this, %pos, %rot, %scale, %dontSelect)
{
	return %this.createPrimitive(CylinderShape, %pos, %rot, %scale, %dontSelect);
}

function BlenderSceneSO::createPlane(%this, %pos, %rot, %scale, %dontSelect)
{
	return %this.createPrimitive(PlaneShape, %pos, %rot, %scale, %dontSelect);
}

function BlenderSceneSO::createPrimitive(%this, %data, %pos, %rot, %scale, %dontSelect)
{
	if (!%dontSelect && isObject(%this.tool) && isObject(%this.tool.user))
		return;
	if (%scale $= "")
		%scale = "1 1 1";
	%shape = new StaticShape()
	{
		datablock = %data;
		scale = vectorScale(%scale, 2);
	};
	if (%data.wireframe !$= "")
		%shape.wireframe = new StaticShape()
		{
			datablock = %data.wireframe;
			scale = vectorScale(%scale, 2);
		};
	else
		%shape.wireframe = %shape;
	%shape.setNodeColor("ALL", "0.5 0.5 0.5 1");
	%shape.setTransform(vectorScale(%pos, 2) SPC %rot);
	%shape.shapeColor = "0.5 0.5 0.5 1";
	%shape.shapePosition = %shape.getPosition();
	%shape.shapeRotation = getWords(%shape.getTransform(), 3, 6);
	%shape.shapeScale = %shape.getScale();
	if (%shape != %shape.wireframe)
	{
		%shape.wireframe.setTransform(%shape.getTransform());
		%shape.wireframe.hideNode("ALL");
		%this.wireframes.add(%shape.wireframe);
	}
	%this.objects.add(%shape);
	if (%dontSelect)
		return %shape;
	%this.selectAll(-1);
	%this.select(%shape);
	return %shape;
}

function BlenderSceneSO::createUVSphere(%this, %pos, %rot, %scale, %dontSelect)
{
	return %this.createPrimitive(UVSphereShape, %pos, %rot, %scale, %dontSelect);
}

function BlenderSceneSO::select(%this, %shape, %shiftSelect)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	if (!%this.objects.isMember(%shape))
		return;
	if (%shiftSelect)
	{
		if (%this.selectedObject == %shape)
		{
			%shape.selected = 0;
			if (%shape.wireframe == %shape)
				%shape.setNodeColor("ALL", "0 0 0 1");
			else
				%shape.wireframe.hideNode("ALL");
			if (isObject(%this.deselectedScene))
				%shape.deselectReturn = %this.deselectedScene.schedule(16, "give", %shape);
			%this.selectedObject = "";
		}
		else
		{
			if (isObject(%this.selectedObject))
				%this.selectedObject.wireframe.setNodeColor("ALL", "0.945 0.345 0 1");
			if (!%shape.selected)
			{
				%shape.selected = 1;
				cancel(%shape.deselectReturn);
				%shape.wireframe.unhideNode("ALL");
			}
			%shape.wireframe.setNodeColor("ALL", "1 0.667 0.251 1");
			%this.selectedObject = %shape;
			%this.lastSelected = %shape;
			if (isObject(%this.tool))
				updateToolTransform(%this.tool);
		}
	}
	else
	{
		%this.selectAll(-1);
		%this.select(%shape, 1);
	}
	if (isObject(%this.tool))
		if (isObject(%this.selectedObject))
			%this.tool.unhideNode("ALL");
		else
			%this.tool.hideNode("ALL");
}

function BlenderSceneSO::destroy(%this)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	%objs = %this.objects;
	%count = %objs.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%obj = %objs.getObject(%i);
		if (!%obj.selected)
			continue;
		%del[%num++ - 1] = %obj;
	}
	for (%i = 0; %i < %num; %i++)
	{
		%obj = %del[%i];
		if (isObject(%obj.wireframe))
			%obj.wireframe.delete();
		if (isObject(%obj))
			%obj.delete();
	}
	if (isObject(%this.tool))
		%this.tool.hideNode("ALL");
}

function BlenderSceneSO::loadDefault(%this)
{
	BlenderScene.createCamera("7.48113 -6.50764 5.34367", "-0.772239 -0.341341 -0.535848 1.34714", 1);
	BlenderScene.createCube("0 0 0", "1 0 0 0", "1 1 1", 1);
}

function BlenderSceneSO::resetScene(%this)
{
	%this.selectAll(1);
	%this.destroy();
	%this.loadDefault();
	%clients = ClientGroup.getCount();
	for (%i = 0; %i < %clients; %i++)
		ClientGroup.getObject(%i).instantRespawn();
}

function BlenderSceneSO::selectAll(%this, %mode)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	%objs = %this.objects;
	%count = %objs.getCount();
	if (%count == 0)
		return;
	if (%mode == 0)
	{
		for (%i = 0; %i < %count; %i++)
		{
			%obj = %objs.getObject(%i);
			if (%obj.selected)
				%selCount++;
		}
		%mode = (%selCount == 0) * 2 - 1;
	}
	if (%mode > 0)
	{
		if (!isObject(%this.selectedObject) && isObject(%this.lastSelected))
			%this.selectedObject = %this.lastSelected;
		for (%i = 0; %i < %count; %i++)
		{
			%obj = %objs.getObject(%i);
			%obj.selected = 1;
			cancel(%obj.deselectReturn());
			%obj.wireframe.unhideNode("ALL");
			%obj.wireframe.setNodeColor("ALL",
				%this.selectedObject == %obj ?
				"1 0.667 0.251 1" : "0.945 0.345 0 1");
		}
		if (isObject(%this.tool))
		{
			%this.tool.unhideNode("ALL");
			updateToolTransform(%this.tool);
		}
	}
	else
	{
		for (%i = 0; %i < %count; %i++)
		{
			%obj = %objs.getObject(%i);
			if (!%obj.selected)
				continue;
			%obj.selected = 0;
			if (isObject(%this.deselectedScene))
				%obj.deselectReturn = %this.deselectedScene.schedule(16, "give", %obj);
			if (%obj.wireframe == %obj)
				%obj.setNodeColor("ALL", "0 0 0 1");
			else
				%obj.wireframe.hideNode("ALL");
		}
		%this.selectedObject = "";
		if (isObject(%this.tool))
			%this.tool.hideNode("ALL");
	}
}

function BlenderSceneSO::selectIndex(%this, %index)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	if (%index < 0 || %index >= %this.objects.getCount())
		return;
	%this.select(%this.objects.getObject(%index));
}

function BlenderSceneSO::setActiveCamera(%this, %camera)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	if (!isObject(%camera) || %this.activeCamera == %camera || !%this.objects.isMember(%camera))
		return;
	%camera.setDatablock(CameraActiveShape);
	%this.activeCamera.setDatablock(CameraInactiveShape);
	for (%i = 0; %i < 2; %i++)
	{
		if (%this.activeCamera.selected)
			if (%this.selectedObject == %this.activeCamera)
				%this.activeCamera.setNodeColor("ALL", "1 0.667 0.251 1");
			else
				%this.activeCamera.setNodeColor("ALL", "0.945 0.345 0 1");
		else
			%this.activeCamera.setNodeColor("ALL", "0 0 0 1");
		if (%i == 0)
			%this.activeCamera = %camera;
	}
}

function BlenderSceneSO::setColor(%this, %shape, %color)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	if (getWordCount(%shape) >= 3)
	{
		%color = %shape;
		if (getWordCount(%color) == 3)
			%color = %color SPC 1;
		%objs = %this.objects;
		%count = %objs.getCount();
		for (%i = 0; %i < %count; %i++)
		{
			%obj = %objs.getObject(%i);
			if (!%obj.selected)
				continue;
			if (%obj.wireframe == %obj)
				continue;
			%obj.shapeColor = %color;
			%obj.setNodeColor("ALL", %color);
		}
		return;
	}
	if (!isObject(%shape))
		return;
	if (%shape.wireframe == %shape)
		return;
	if (getWordCount(%color) == 3)
		%color = %color SPC 1;
	%shape.shapeColor = %color;
	%shape.setNodeColor("ALL", %color);
}

function BlenderSceneSO::setShading(%this, %smooth)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	%objs = %this.objects;
	%count = %objs.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%obj = %objs.getObject(%i);
		if (!%obj.selected)
			continue;
		if (%obj.wireframe == %obj)
			continue;
		%data = %obj.getDatablock();
		if (%smooth)
		{
			if (isObject(%data.smoothData))
			{
				%obj.setDatablock(%data.smoothData);
				%obj.setNodeColor("ALL", %obj.shapeColor);
			}
		}
		else
		{
			if (isObject(%data.flatData))
			{
				%obj.setDatablock(%data.flatData);
				%obj.setNodeColor("ALL", %obj.shapeColor);
			}
		}
	}
}

function BlenderSceneSO::setTool(%this, %mode)
{
	if (isObject(%this.tool) && isObject(%this.tool.user))
		return;
	switch$(%mode)
	{
		case "translate":
			%this.tool.currMode = "translate";
			%this.tool.setDatablock(ToolTranslateShape);
			%this.tool.setNodeColor("TranslateX", "0.863 0 0 1");
			%this.tool.setNodeColor("TranslateY", "0 0.863 0 1");
			%this.tool.setNodeColor("TranslateZ", "0 0 0.863 1");
		case "rotate":
			%this.tool.currMode = "rotate";
			%this.tool.setDatablock(ToolRotateShape);
			%this.tool.setNodeColor("RotateX", "0.863 0 0 1");
			%this.tool.setNodeColor("RotateY", "0 0.863 0 1");
			%this.tool.setNodeColor("RotateZ", "0 0 0.863 1");
		case "scale":
			%this.tool.currMode = "scale";
			%this.tool.setDatablock(ToolScaleShape);
			%this.tool.setNodeColor("ScaleX", "0.863 0 0 1");
			%this.tool.setNodeColor("ScaleY", "0 0.863 0 1");
			%this.tool.setNodeColor("ScaleZ", "0 0 0.863 1");
	}
	if (!isObject(%this.selectedObject))
		%this.tool.hideNode("ALL");
}

function BlenderSceneSO::give(%this, %shape)
{
	%this.objects.add(%shape);
	if (%shape.wireframe != %shape)
		%this.wireframes.add(%shape.wireframe);
}

function BlenderSceneSO::request(%this, %requestor, %shape)
{
	if (!%this.objects.isMember(%shape))
		return 0;
	else if (isFunction(%this.requestCallback))
	{
		%response = call(%this.requestCallback, %requestor, %shape);
		if (%response)
			%requestor.give(%shape);
		return %response;
	}
	else
		return 0;
}

function BlenderSceneSO::delete(%this)
{
	%this.objects.delete();
	%this.wireframes.delete();
	if (isObject(%this.grid))
		%this.grid.delete();
	%this.tool.delete();
	Parent::delete(%this);
}